@@ -17,9 +17,8 @@ from typing import (
 )
 
 import tomli
-from dataclasses_json import DataClassJsonMixin
-from pydantic import Field
-from pydantic.dataclasses import dataclass
+from pydantic import BaseModel, Field
+from pydantic_settings import BaseSettings
 from starlette.datastructures import Headers
 
 from chainlit.data.base import BaseDataLayer
@@ -79,6 +78,14 @@ user_session_timeout = 1296000  # 15 days
 # Enable third parties caching (e.g., LangChain cache)
 cache = false
 
+# Whether to persist user environment variables (API keys) to the database
+# Set to true to store user env vars in DB, false to exclude them for security
+persist_user_env = false
+
+# Whether to mask user environment variables (API keys) in the UI with password type
+# Set to true to show API keys as ***, false to show them as plain text
+mask_user_env = false
+
 # Authorized origins
 allow_origins = ["*"]
 
@@ -98,6 +105,9 @@ auto_tag_thread = true
 # Allow users to edit their own messages
 edit_message = true
 
+# Allow users to share threads (backend + UI). Requires an app-defined on_shared_thread_view callback.
+allow_thread_sharing = false
+
 [features.slack]
 # Add emoji reaction when message is received (requires reactions:write OAuth scope)
 reaction_on_message_received = false
@@ -119,9 +129,15 @@ reaction_on_message_received = false
     max_size_mb = 500
 
 [features.audio]
+    # Enable audio features
+    enabled = false
     # Sample rate of the audio
     sample_rate = 24000
 
+[features.mcp]
+    # Enable Model Context Protocol (MCP) features
+    enabled = false
+
 [features.mcp.sse]
     enabled = true
 
@@ -207,8 +223,7 @@ DEFAULT_PORT = 8000
 DEFAULT_ROOT_PATH = ""
 
 
-@dataclass()
-class RunSettings:
+class RunSettings(BaseModel):
     # Name of the module (python file) used in the run command
     module_name: Optional[str] = None
     host: str = DEFAULT_HOST
@@ -223,64 +238,54 @@ class RunSettings:
     ci: bool = False
 
 
-@dataclass()
-class PaletteOptions(DataClassJsonMixin):
+class PaletteOptions(BaseModel):
     main: Optional[str] = ""
     light: Optional[str] = ""
     dark: Optional[str] = ""
 
 
-@dataclass()
-class TextOptions(DataClassJsonMixin):
+class TextOptions(BaseModel):
     primary: Optional[str] = ""
     secondary: Optional[str] = ""
 
 
-@dataclass()
-class Palette(DataClassJsonMixin):
+class Palette(BaseModel):
     primary: Optional[PaletteOptions] = None
     background: Optional[str] = ""
     paper: Optional[str] = ""
     text: Optional[TextOptions] = None
 
 
-@dataclass
-class SpontaneousFileUploadFeature(DataClassJsonMixin):
+class SpontaneousFileUploadFeature(BaseModel):
     enabled: Optional[bool] = None
     accept: Optional[Union[List[str], Dict[str, List[str]]]] = None
     max_files: Optional[int] = None
     max_size_mb: Optional[int] = None
 
 
-@dataclass
-class AudioFeature(DataClassJsonMixin):
+class AudioFeature(BaseModel):
     sample_rate: int = 24000
     enabled: bool = False
 
 
-@dataclass
-class McpSseFeature(DataClassJsonMixin):
+class McpSseFeature(BaseModel):
     enabled: bool = True
 
 
-@dataclass
-class McpStreamableHttpFeature(DataClassJsonMixin):
+class McpStreamableHttpFeature(BaseModel):
     enabled: bool = True
 
 
-@dataclass
-class McpStdioFeature(DataClassJsonMixin):
+class McpStdioFeature(BaseModel):
     enabled: bool = True
     allowed_executables: Optional[list[str]] = None
 
 
-@dataclass
-class SlackFeature(DataClassJsonMixin):
+class SlackFeature(BaseModel):
     reaction_on_message_received: bool = False
 
 
-@dataclass
-class McpFeature(DataClassJsonMixin):
+class McpFeature(BaseModel):
     enabled: bool = False
     sse: McpSseFeature = Field(default_factory=McpSseFeature)
     streamable_http: McpStreamableHttpFeature = Field(
@@ -289,8 +294,7 @@ class McpFeature(DataClassJsonMixin):
     stdio: McpStdioFeature = Field(default_factory=McpStdioFeature)
 
 
-@dataclass()
-class FeaturesSettings(DataClassJsonMixin):
+class FeaturesSettings(BaseModel):
     spontaneous_file_upload: Optional[SpontaneousFileUploadFeature] = None
     audio: Optional[AudioFeature] = Field(default_factory=AudioFeature)
     mcp: McpFeature = Field(default_factory=McpFeature)
@@ -300,18 +304,17 @@ class FeaturesSettings(DataClassJsonMixin):
     unsafe_allow_html: bool = False
     auto_tag_thread: bool = True
     edit_message: bool = True
+    allow_thread_sharing: bool = False
 
 
-@dataclass
-class HeaderLink(DataClassJsonMixin):
+class HeaderLink(BaseModel):
     name: str
     icon_url: str
     url: str
     display_name: Optional[str] = None
 
 
-@dataclass()
-class UISettings(DataClassJsonMixin):
+class UISettings(BaseModel):
     name: str
     description: str = ""
     cot: Literal["hidden", "tool_call", "full"] = "full"
@@ -344,8 +347,7 @@ class UISettings(DataClassJsonMixin):
     header_links: Optional[List[HeaderLink]] = None
 
 
-@dataclass()
-class CodeSettings:
+class CodeSettings(BaseModel):
     # App action functions
     action_callbacks: Dict[str, Callable[["Action"], Any]]
 
@@ -371,11 +373,14 @@ class CodeSettings:
     on_mcp_disconnect: Optional[Callable] = None
     on_settings_update: Optional[Callable[[Dict[str, Any]], Any]] = None
     set_chat_profiles: Optional[
-        Callable[[Optional["User"]], Awaitable[List["ChatProfile"]]]
+        Callable[[Optional["User"], Optional["str"]], Awaitable[List["ChatProfile"]]]
+    ] = None
+    set_starters: Optional[
+        Callable[[Optional["User"], Optional["str"]], Awaitable[List["Starter"]]]
+    ] = None
+    on_shared_thread_view: Optional[
+        Callable[["ThreadDict", Optional["User"]], Awaitable[bool]]
     ] = None
-    set_starters: Optional[Callable[[Optional["User"]], Awaitable[List["Starter"]]]] = (
-        None
-    )
     # Auth callbacks
     password_auth_callback: Optional[
         Callable[[str, str], Awaitable[Optional["User"]]]
@@ -393,8 +398,7 @@ class CodeSettings:
     data_layer: Optional[Callable[[], BaseDataLayer]] = None
 
 
-@dataclass()
-class ProjectSettings(DataClassJsonMixin):
+class ProjectSettings(BaseModel):
     allow_origins: List[str] = Field(default_factory=lambda: ["*"])
     # Socket.io client transports option
     transports: Optional[List[str]] = None
@@ -409,15 +413,24 @@ class ProjectSettings(DataClassJsonMixin):
     user_session_timeout: int = 1296000  # 15 days
     # Enable third parties caching (e.g LangChain cache)
     cache: bool = False
+    # Whether to persist user environment variables (API keys) to the database
+    persist_user_env: Optional[bool] = False
+    # Whether to mask user environment variables (API keys) in the UI with password type
+    mask_user_env: Optional[bool] = False
 
 
-@dataclass()
-class ChainlitConfig:
-    # Directory where the Chainlit project is located
-    root = APP_ROOT
-    # Chainlit server URL. Used only for cloud features
-    chainlit_server: str
-    run: RunSettings
+class ChainlitConfigOverrides(BaseModel):
+    """Configuration overrides that can be applied to specific chat profiles."""
+
+    ui: Optional[UISettings] = None
+    features: Optional[FeaturesSettings] = None
+    project: Optional[ProjectSettings] = None
+
+
+class ChainlitConfig(BaseSettings):
+    root: str = APP_ROOT
+    chainlit_server: str = Field(default="")
+    run: RunSettings = Field(default_factory=RunSettings)
     features: FeaturesSettings
     ui: UISettings
     project: ProjectSettings
@@ -467,8 +480,25 @@ class ChainlitConfig:
 
         return translation
 
+    def with_overrides(
+        self, overrides: "ChainlitConfigOverrides | None"
+    ) -> "ChainlitConfig":
+        base = self.model_dump()
+        patch = overrides.model_dump(exclude_unset=True) if overrides else {}
 
-def init_config(log=False):
+        def _merge(a, b):
+            if isinstance(a, dict) and isinstance(b, dict):
+                out = dict(a)
+                for k, v in b.items():
+                    out[k] = _merge(out.get(k), v)
+                return out
+            return b
+
+        merged = _merge(base, patch) if patch else base
+        return type(self).model_validate(merged)
+
+
+def init_config(log: bool = False):
     """Initialize the configuration file if it doesn't exist."""
     if not os.path.exists(config_file):
         os.makedirs(config_dir, exist_ok=True)
@@ -521,10 +551,12 @@ def load_module(target: str, force_refresh: bool = False):
 
     spec = util.spec_from_file_location(target, target)
     if not spec or not spec.loader:
+        sys.path.pop(0)
         return
 
     module = util.module_from_spec(spec)
     if not module:
+        sys.path.pop(0)
         return
 
     spec.loader.exec_module(module)
@@ -576,29 +608,28 @@ def reload_config():
     if config is None:
         return
 
-    settings = load_settings()
+    # Preserve the module_name during config reload to ensure hot reload works
+    original_module_name = config.run.module_name if config.run else None
 
-    config.features = settings["features"]
-    config.code = settings["code"]
-    config.ui = settings["ui"]
-    config.project = settings["project"]
+    new_cfg = ChainlitConfig(**load_settings())
+    config.root = new_cfg.root
+    config.chainlit_server = new_cfg.chainlit_server
+    config.run = new_cfg.run
+    config.features = new_cfg.features
+    config.ui = new_cfg.ui
+
+    # Restore the preserved module_name
+    if original_module_name and config.run:
+        config.run.module_name = original_module_name
+    config.project = new_cfg.project
+    config.code = new_cfg.code
 
 
 def load_config():
     """Load the configuration from the config file."""
     init_config()
-
     settings = load_settings()
-
-    chainlit_server = os.environ.get("CHAINLIT_SERVER", "https://cloud.chainlit.io")
-
-    config = ChainlitConfig(
-        chainlit_server=chainlit_server,
-        run=RunSettings(),
-        **settings,
-    )
-
-    return config
+    return ChainlitConfig(**settings)
 
 
 def lint_translations():
@@ -612,8 +643,8 @@ def lint_translations():
             if file.endswith(".json"):
                 # Load the translation file
                 to_lint = os.path.join(config_translation_dir, file)
-                with open(to_lint, encoding="utf-8") as f:
-                    translation = json.load(f)
+                with open(to_lint, encoding="utf-8") as f2:
+                    translation = json.load(f2)
 
                     # Lint the translation file
                     lint_translation_json(file, truth, translation)
