@@ -30,6 +30,7 @@ from fastapi.responses import FileResponse, HTMLResponse, JSONResponse, Redirect
 from fastapi.security import OAuth2PasswordRequestForm
 from starlette.datastructures import URL
 from starlette.middleware.cors import CORSMiddleware
+from starlette.types import Receive, Scope, Send
 from typing_extensions import Annotated
 from watchfiles import awatch
 
@@ -47,6 +48,7 @@ from chainlit.config import (
     DEFAULT_HOST,
     FILES_DIRECTORY,
     PACKAGE_ROOT,
+    ChainlitConfig,
     config,
     load_module,
     public_dir,
@@ -59,6 +61,7 @@ from chainlit.markdown import get_markdown_str
 from chainlit.oauth_providers import get_oauth_provider
 from chainlit.secret import random_secret
 from chainlit.types import (
+    AskFileSpec,
     CallActionRequest,
     ConnectMCPRequest,
     DeleteFeedbackRequest,
@@ -66,11 +69,13 @@ from chainlit.types import (
     DisconnectMCPRequest,
     ElementRequest,
     GetThreadsRequest,
+    ShareThreadRequest,
     Theme,
     UpdateFeedbackRequest,
     UpdateThreadRequest,
 )
 from chainlit.user import PersistedUser, User
+from chainlit.utils import utc_now
 
 from ._utils import is_path_inside
 
@@ -145,6 +150,14 @@ async def lifespan(app: FastAPI):
 
         discord_task = asyncio.create_task(client.start(discord_bot_token))
 
+    slack_task = None
+
+    # Slack Socket Handler if env variable SLACK_WEBSOCKET_TOKEN is set
+    if os.environ.get("SLACK_BOT_TOKEN") and os.environ.get("SLACK_WEBSOCKET_TOKEN"):
+        from chainlit.slack.app import start_socket_mode
+
+        slack_task = asyncio.create_task(start_socket_mode())
+
     try:
         yield
     finally:
@@ -160,6 +173,10 @@ async def lifespan(app: FastAPI):
             if discord_task:
                 discord_task.cancel()
                 await discord_task
+
+            if slack_task:
+                slack_task.cancel()
+                await slack_task
         except asyncio.exceptions.CancelledError:
             pass
 
@@ -207,7 +224,8 @@ sio = socketio.AsyncServer(cors_allowed_origins=[], async_mode="asgi")
 asgi_app = socketio.ASGIApp(socketio_server=sio, socketio_path="")
 
 # config.run.root_path is only set when started with --root-path. Not on submounts.
-app.mount(f"{config.run.root_path}/ws/socket.io", asgi_app)
+SOCKET_IO_PATH = f"{config.run.root_path}/ws/socket.io"
+app.mount(SOCKET_IO_PATH, asgi_app)
 
 app.add_middleware(
     CORSMiddleware,
@@ -217,7 +235,20 @@ app.add_middleware(
     allow_headers=["*"],
 )
 
-app.add_middleware(GZipMiddleware)
+
+class SafariWebSocketsCompatibleGZipMiddleware(GZipMiddleware):
+    async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
+        if scope["type"] != "http":
+            return await self.app(scope, receive, send)
+
+        # Prevent gzip compression for HTTP requests to socket.io path due to a bug in Safari
+        if URL(scope=scope).path.startswith(SOCKET_IO_PATH):
+            await self.app(scope, receive, send)
+        else:
+            await super().__call__(scope, receive, send)
+
+
+app.add_middleware(SafariWebSocketsCompatibleGZipMiddleware)
 
 # config.run.root_path is only set when started with --root-path. Not on submounts.
 router = APIRouter(prefix=config.run.root_path)
@@ -278,10 +309,14 @@ async def serve_copilot_file(
 
 
 # -------------------------------------------------------------------------------
-#                               SLACK HANDLER
+#                               SLACK HTTP HANDLER
 # -------------------------------------------------------------------------------
 
-if os.environ.get("SLACK_BOT_TOKEN") and os.environ.get("SLACK_SIGNING_SECRET"):
+if (
+    os.environ.get("SLACK_BOT_TOKEN")
+    and os.environ.get("SLACK_SIGNING_SECRET")
+    and not os.environ.get("SLACK_WEBSOCKET_TOKEN")
+):
     from chainlit.slack.app import slack_app_handler
 
     @router.post("/slack/events")
@@ -762,44 +797,57 @@ async def project_settings(
     language: str = Query(
         default="en-US", description="Language code", pattern=_language_pattern
     ),
+    chat_profile: Optional[str] = Query(
+        default=None, description="Current chat profile name"
+    ),
 ):
     """Return project settings. This is called by the UI before the establishing the websocket connection."""
 
     # Load the markdown file based on the provided language
-
     markdown = get_markdown_str(config.root, language)
 
-    profiles = []
+    chat_profiles = []
+    profiles: list[dict] = []
     if config.code.set_chat_profiles:
-        chat_profiles = await config.code.set_chat_profiles(current_user)
+        chat_profiles = await config.code.set_chat_profiles(current_user, language)
         if chat_profiles:
-            profiles = [p.to_dict() for p in chat_profiles]
+            for p in chat_profiles:
+                d = p.to_dict()
+                d.pop("config_overrides", None)
+                profiles.append(d)
 
     starters = []
     if config.code.set_starters:
-        starters = await config.code.set_starters(current_user)
-        if starters:
-            starters = [s.to_dict() for s in starters]
+        s = await config.code.set_starters(current_user, language)
+        if s:
+            starters = [it.to_dict() for it in s]
 
-    if config.code.on_audio_chunk:
-        config.features.audio.enabled = True
-
-    if config.code.on_mcp_connect:
-        config.features.mcp.enabled = True
-
-    debug_url = None
     data_layer = get_data_layer()
+    debug_url = (
+        await data_layer.build_debug_url() if data_layer and config.run.debug else None
+    )
 
-    if data_layer and config.run.debug:
-        debug_url = await data_layer.build_debug_url()
+    cfg = config
+    if chat_profile and chat_profiles:
+        current_profile = next(
+            (p for p in chat_profiles if p.name == chat_profile), None
+        )
+        if current_profile and getattr(current_profile, "config_overrides", None):
+            cfg = config.with_overrides(current_profile.config_overrides)
 
     return JSONResponse(
         content={
-            "ui": config.ui.to_dict(),
-            "features": config.features.to_dict(),
-            "userEnv": config.project.user_env,
-            "dataPersistence": get_data_layer() is not None,
+            "ui": cfg.ui.model_dump(),
+            "features": cfg.features.model_dump(),
+            "userEnv": cfg.project.user_env,
+            "maskUserEnv": cfg.project.mask_user_env,
+            "dataPersistence": data_layer is not None,
             "threadResumable": bool(config.code.on_chat_resume),
+            # Expose whether shared threads feature is enabled (flag + app callback)
+            "threadSharing": bool(
+                getattr(cfg.features, "allow_thread_sharing", False)
+                and getattr(config.code, "on_shared_thread_view", None)
+            ),
             "markdown": markdown,
             "chatProfiles": profiles,
             "starters": starters,
@@ -824,6 +872,12 @@ async def update_feedback(
 
         if config.code.on_feedback:
             try:
+                from chainlit.context import init_ws_context
+                from chainlit.session import WebsocketSession
+
+                session = WebsocketSession.get_by_id(update.sessionId)
+                init_ws_context(session)
+
                 await config.code.on_feedback(update.feedback)
             except Exception as callback_error:
                 logger.error(
@@ -904,6 +958,60 @@ async def get_thread(
     return JSONResponse(content=res)
 
 
+@router.get("/project/share/{thread_id}")
+async def get_shared_thread(
+    request: Request,
+    thread_id: str,
+    current_user: UserParam,
+):
+    """Get a shared thread (read-only for everyone).
+
+    This endpoint is separate from the resume endpoint and does not require the caller
+    to be the author of the thread. It only returns the thread if its metadata
+    contains is_shared=True. Otherwise, it returns 404 to avoid leaking existence.
+    """
+
+    data_layer = get_data_layer()
+
+    if not data_layer:
+        raise HTTPException(status_code=400, detail="Data persistence is not enabled")
+
+    # No auth required: allow anonymous access to shared threads
+    thread = await data_layer.get_thread(thread_id)
+
+    if not thread:
+        raise HTTPException(status_code=404, detail="Thread not found")
+    # Extract and normalize metadata (may be dict, strified JSON, or None)
+    metadata = (thread.get("metadata") if isinstance(thread, dict) else {}) or {}
+    if isinstance(metadata, str):
+        try:
+            metadata = json.loads(metadata)
+        except Exception:
+            metadata = {}
+    if not isinstance(metadata, dict):
+        metadata = {}
+
+    if getattr(config.code, "on_shared_thread_view", None):
+        try:
+            user_can_view = await config.code.on_shared_thread_view(
+                thread, current_user
+            )
+        except Exception:
+            user_can_view = False
+
+    is_shared = bool(metadata.get("is_shared"))
+
+    # Proceed only if both conditions are True.
+    if not (user_can_view and is_shared):
+        raise HTTPException(status_code=404, detail="Thread not found")
+
+    metadata.pop("chat_profile", None)
+    metadata.pop("chat_settings", None)
+    metadata.pop("env", None)
+    thread["metadata"] = metadata
+    return JSONResponse(content=thread)
+
+
 @router.get("/project/thread/{thread_id}/element/{element_id}")
 async def get_thread_element(
     request: Request,
@@ -1028,6 +1136,59 @@ async def rename_thread(
     await is_thread_author(current_user.identifier, thread_id)
 
     await data_layer.update_thread(thread_id, name=payload.name)
+
+    return JSONResponse(content={"success": True})
+
+
+@router.put("/project/thread/share")
+async def share_thread(
+    request: Request,
+    payload: ShareThreadRequest,
+    current_user: UserParam,
+):
+    """Share or un-share a thread (author only)."""
+
+    data_layer = get_data_layer()
+
+    if not data_layer:
+        raise HTTPException(status_code=400, detail="Data persistence is not enabled")
+
+    if not current_user:
+        raise HTTPException(status_code=401, detail="Unauthorized")
+
+    thread_id = payload.threadId
+
+    await is_thread_author(current_user.identifier, thread_id)
+
+    # Fetch current thread and metadata, then toggle is_shared
+    thread = await data_layer.get_thread(thread_id=thread_id)
+    metadata = (thread.get("metadata") if thread else {}) or {}
+    if isinstance(metadata, str):
+        try:
+            metadata = json.loads(metadata)
+        except Exception:
+            metadata = {}
+    if not isinstance(metadata, dict):
+        metadata = {}
+
+    metadata = dict(metadata)
+    is_shared = bool(payload.isShared)
+    metadata["is_shared"] = is_shared
+    if is_shared:
+        metadata["shared_at"] = utc_now()
+    else:
+        metadata.pop("shared_at", None)
+    try:
+        await data_layer.update_thread(thread_id=thread_id, metadata=metadata)
+        logger.debug(
+            "[share_thread] updated metadata for thread=%s to %s",
+            thread_id,
+            metadata,
+        )
+    except Exception as e:
+        logger.exception("[share_thread] update_thread failed: %s", e)
+        raise
+
     return JSONResponse(content={"success": True})
 
 
@@ -1068,6 +1229,7 @@ async def call_action(
 
     session = WebsocketSession.get_by_id(payload.sessionId)
     context = init_ws_context(session)
+    config: ChainlitConfig = session.get_config()
 
     action = Action(**payload.action)
 
@@ -1123,6 +1285,7 @@ async def connect_mcp(
 
     session = WebsocketSession.get_by_id(payload.sessionId)
     context = init_ws_context(session)
+    config: ChainlitConfig = session.get_config()
 
     if current_user:
         if (
@@ -1133,7 +1296,7 @@ async def connect_mcp(
                 status_code=401,
             )
 
-    mcp_enabled = config.code.on_mcp_connect is not None
+    mcp_enabled = config.features.mcp.enabled
     if mcp_enabled:
         if payload.name in session.mcp_sessions:
             old_client_session, old_exit_stack = session.mcp_sessions[payload.name]
@@ -1226,7 +1389,8 @@ async def connect_mcp(
             session.mcp_sessions[mcp_connection.name] = (mcp_session, exit_stack)
 
             # Call the callback
-            await config.code.on_mcp_connect(mcp_connection, mcp_session)
+            if config.code.on_mcp_connect:
+                await config.code.on_mcp_connect(mcp_connection, mcp_session)
 
         except Exception as e:
             raise HTTPException(
@@ -1310,6 +1474,7 @@ async def upload_file(
     current_user: UserParam,
     session_id: str,
     file: UploadFile,
+    ask_parent_id: Optional[str] = None,
 ):
     """Upload a file to the session files directory."""
 
@@ -1338,8 +1503,15 @@ async def upload_file(
         assert file.filename, "No filename for uploaded file"
         assert file.content_type, "No content type for uploaded file"
 
+        spec: AskFileSpec = session.files_spec.get(ask_parent_id, None)
+        if not spec and ask_parent_id:
+            raise HTTPException(
+                status_code=404,
+                detail="Parent message not found",
+            )
+
         try:
-            validate_file_upload(file)
+            validate_file_upload(file, spec=spec)
         except ValueError as e:
             raise HTTPException(status_code=400, detail=str(e))
 
@@ -1352,27 +1524,28 @@ async def upload_file(
         await file.close()
 
 
-def validate_file_upload(file: UploadFile):
-    """Validate the file upload as configured in config.features.spontaneous_file_upload.
+def validate_file_upload(file: UploadFile, spec: Optional[AskFileSpec] = None):
+    """Validate the file upload as configured in config.features.spontaneous_file_upload or by AskFileSpec
+    for a specific message.
+
     Args:
         file (UploadFile): The file to validate.
+        spec (AskFileSpec): The file spec to validate against if any.
     Raises:
         ValueError: If the file is not allowed.
     """
-    # TODO: This logic/endpoint is shared across spontaneous uploads and the AskFileMessage API.
-    # Commenting this check until we find a better solution
+    if not spec and config.features.spontaneous_file_upload is None:
+        """Default for a missing config is to allow the fileupload without any restrictions"""
+        return
 
-    # if config.features.spontaneous_file_upload is None:
-    #     """Default for a missing config is to allow the fileupload without any restrictions"""
-    #     return
-    # if not config.features.spontaneous_file_upload.enabled:
-    #     raise ValueError("File upload is not enabled")
+    if not spec and not config.features.spontaneous_file_upload.enabled:
+        raise ValueError("File upload is not enabled")
 
-    validate_file_mime_type(file)
-    validate_file_size(file)
+    validate_file_mime_type(file, spec)
+    validate_file_size(file, spec)
 
 
-def validate_file_mime_type(file: UploadFile):
+def validate_file_mime_type(file: UploadFile, spec: Optional[AskFileSpec]):
     """Validate the file mime type as configured in config.features.spontaneous_file_upload.
     Args:
         file (UploadFile): The file to validate.
@@ -1380,14 +1553,14 @@ def validate_file_mime_type(file: UploadFile):
         ValueError: If the file type is not allowed.
     """
 
-    if (
+    if not spec and (
         config.features.spontaneous_file_upload is None
         or config.features.spontaneous_file_upload.accept is None
     ):
         "Accept is not configured, allowing all file types"
         return
 
-    accept = config.features.spontaneous_file_upload.accept
+    accept = config.features.spontaneous_file_upload.accept if not spec else spec.accept
 
     assert isinstance(accept, List) or isinstance(accept, dict), (
         "Invalid configuration for spontaneous_file_upload, accept must be a list or a dict"
@@ -1395,11 +1568,11 @@ def validate_file_mime_type(file: UploadFile):
 
     if isinstance(accept, List):
         for pattern in accept:
-            if fnmatch.fnmatch(file.content_type, pattern):
+            if fnmatch.fnmatch(str(file.content_type), pattern):
                 return
     elif isinstance(accept, dict):
         for pattern, extensions in accept.items():
-            if fnmatch.fnmatch(file.content_type, pattern):
+            if fnmatch.fnmatch(str(file.content_type), pattern):
                 if len(extensions) == 0:
                     return
                 for extension in extensions:
@@ -1410,24 +1583,25 @@ def validate_file_mime_type(file: UploadFile):
     raise ValueError("File type not allowed")
 
 
-def validate_file_size(file: UploadFile):
+def validate_file_size(file: UploadFile, spec: Optional[AskFileSpec]):
     """Validate the file size as configured in config.features.spontaneous_file_upload.
     Args:
         file (UploadFile): The file to validate.
     Raises:
         ValueError: If the file size is too large.
     """
-    if (
+    if not spec and (
         config.features.spontaneous_file_upload is None
         or config.features.spontaneous_file_upload.max_size_mb is None
     ):
         return
 
-    if (
-        file.size is not None
-        and file.size
-        > config.features.spontaneous_file_upload.max_size_mb * 1024 * 1024
-    ):
+    max_size_mb = (
+        config.features.spontaneous_file_upload.max_size_mb
+        if not spec
+        else spec.max_size_mb
+    )
+    if file.size is not None and file.size > max_size_mb * 1024 * 1024:
         raise ValueError("File size too large")
 
 
